open Ast
exception Unimplemented

(* a pretty-printer client makes one of these *)
type doc =
    DocNil
  | DocText of string
  | DocBreak of string
  | DocCons of doc * doc
  | DocGroup of doc
  | DocNest of int * doc

(* smartcons combines two documents if they're both non-empty *)
val smartcons : doc -> doc -> doc
(* conslist turns an ML list of docs into a "doc list" using DocCons *)
val conslist : doc list -> doc
(* the empty doc *)
val empty : doc
(* turns a piece of text into a doc *)
val text : string -> doc
(* nests the provided list of docs by some amount *)
val nest : int -> doc list -> doc
(* an optional line break *)
val br : doc
(* an optional line break OR the provided string, when in flat mode *)
val breakWith : string -> doc
(* groups the provided list of docs *)
val group : doc list -> doc

(* the format function makes one of these from a doc *)
type sdoc =
    | SNil
    | SText of string * sdoc
    | SLine of int    * sdoc    (* newline + spaces *)

type mode =
    | Flat           (* All the text can fit on one flat line *)
    | Break          (* DocBreaks will become line brs *)

val sdocToString : sdoc -> string
val fits : int -> (int * mode * doc) list -> bool
val format : int -> int -> (int * mode * doc) list -> sdoc
val pretty : int -> doc -> string

val stmtToDoc : stmt -> doc
